/*******************************************************************************
 *	BDebug.h	Debug information and routines
 *			T.Barnaby,	BEAM Ltd,	2009-01-05
 *	Copyright (c) 2012 All Right Reserved, Beam Ltd, http://www.beam.ltd.uk
 *******************************************************************************
 */
#ifndef BDebug_H
#define	BDebug_H

#include <stdio.h>
#include <time.h>

#define	BDebug_STD		0x000001

extern int	bdebug;

void bhd8(const void* data, unsigned int n);
void bhd8a(const void* data, unsigned int n);
void bhda8(const void* data, unsigned int n);
void bhd32(const void* data, unsigned int n);
void bhds32(const void* data, unsigned int n);
double getTime();
void setDebug(int debug);
void tprintf(int log, const char* fmt, ...);

pid_t gettid();

/// General debug functions
#if BDEBUG
#define	dprintf(level, fmt, a...)       if((level) & bdebug) tprintf(1, fmt, ##a);
#else
#define	dprintf(level, fmt, a...)
#endif

/// Warnings and errors logging
#if !TARGET_win32 && !TARGET_win64
#include <syslog.h>

#if BDEBUG
#define	nprintf(fmt, a...)	{ syslog(LOG_NOTICE, fmt, ##a); tprintf(0, fmt, ##a); }
#define	wprintf(fmt, a...)	{ syslog(LOG_WARNING, fmt, ##a); tprintf(0, fmt, ##a); }
#define	eprintf(fmt, a...)	{ syslog(LOG_ERR, fmt, ##a); tprintf(0, fmt, ##a); }
#else
#define	nprintf(fmt, a...)	syslog(LOG_NOTICE, fmt, ##a)
#define	wprintf(fmt, a...)	syslog(LOG_WARNING, fmt, ##a)
#define	eprintf(fmt, a...)	syslog(LOG_ERR, fmt, ##a)
#endif
#endif

#if BDEBUGL1
#define dl1printf(fmt, a...)	printf(fmt, ##a);
#else
#define dl1printf(fmt, a...)
#endif

#if BDEBUGL2
#define dl2printf(fmt, a...)	printf(fmt, ##a);
#else
#define dl2printf(fmt, a...)
#endif

#if BDEBUGL3
#define dl3printf(fmt, a...)	printf(fmt, ##a);
#else
#define dl3printf(fmt, a...)
#endif

#if BDEBUGL4
#define dl4printf(fmt, a...)	printf(fmt, ##a);
#else
#define dl4printf(fmt, a...)
#endif

class BDebugBacktrace {
public:
			BDebugBacktrace();
			~BDebugBacktrace();

	void		dumpBacktraceStdout(char* comment);
	int		dumpBacktraceFile(char* fileName, char* comment);
	void		dumpBacktraceSyslog(char* comment);
	void		dumpBacktrace(char* strBuf, int strBufLen, char* comment);
private:
};

#endif
