/*******************************************************************************
 *	BError.h	BError Class
 *			T.Barnaby,	BEAM Ltd,	21/3/00
 *	Copyright (c) 2012 All Right Reserved, Beam Ltd, http://www.beam.ltd.uk
 *******************************************************************************
 */
#ifndef BERROR_H
#define BERROR_H	1

#include <BString.h>

enum BErrorNum {
	ErrorOk = 0, ErrorMisc = 1, ErrorWarning = 2, ErrorParam = 3, ErrorTimeout = 4,
	ErrorNotAvailable = 5, ErrorData = 6, ErrorChecksum = 7, ErrorOverrun = 8, ErrorUnderrun = 9,
	ErrorInit = 10, ErrorConfig = 11, ErrorNotImplemented = 12, ErrorResourceLimit = 13,
	ErrorEndOfFile = 14, ErrorFile = 15, ErrorFormat = 16, ErrorComms = 17, ErrorAccessDenied = 18,
	ErrorNoData = 19, ErrorEndOfData = 20, ErrorDataPresent = 21,
	ErrorAppBase = 64 };

/// Error return class. This class is used to return the error status from a function. It encapsulates an integer error number and a string.
/// An error number of ErrorOk (0) indicates no error, a value of ErrorMisc (1) indicates some error and a value of ErrorWarning (2)
/// indiactes a warning. Specific error numbers are defeined in BErrorNum. System low level errors (errno) are defined by negativer values.
/// Specific application errors are those above the value 64.
class BError {
public:
			BError(int errNo = ErrorOk, BString errStr = "");	///< Create object
			BError(BString errStr);				///< Create with error set and error string

	BError		copy();						///< Return an independant copy
				
	BError&		set(int errNo, BString errStr = "");		///< Set error number and message
	BError&		clear();					///< Clear the error
	BError&		setError(BString errStr = "");			///< Set error type ERROR with optional message


	BString		getString() const;				///< Get error message
	int		getNumber() const;				///< Get The error number

	int		num() const;					///< Get The error number
	const char*	str() const;					///< Return a char* string
	
	// Old functions
	int		getErrorNo() const;				///< Get The error number

			operator int() const;				///< Return error number
private:
	int		oerrNo;
	BString		oerrStr;
	
};

inline BError::operator int () const {
	return oerrNo;
}

#endif
