/*******************************************************************************
 *	BPoll.h		File poll class
 *			T.Barnaby,	BEAM Ltd,	01/4/05
 *	Copyright (c) 2012 All Right Reserved, Beam Ltd, http://www.beam.ltd.uk
 *******************************************************************************
 */
#ifndef BPOLL_H
#define BPOLL_H	1

#include <BList.h>
#include <BError.h>

#if TARGET_win32 || TARGET_win64
//#undef _WIN32_WINNT
//#define _WIN32_WINNT	WindowsVista
#include <winsock2.h>
#define POLLRDNORM 0x0100
#define POLLRDBAND 0x0200
#define POLLIN    (POLLRDNORM | POLLRDBAND)
#define POLLPRI    0x0400

#define POLLWRNORM 0x0010
#define POLLOUT   (POLLWRNORM)
#define POLLWRBAND 0x0020

#define POLLERR    0x0001
#define POLLHUP    0x0002
#define POLLNVAL   0x0004
#define NFDBITS		(8 * sizeof(int))

struct pollfd {
	int		fd;		/* File descriptor to poll.  */
	short int	events;		/* Types of events poller cares about.  */
	short int	revents;	/* Types of events that actually occurred.  */
};

#else
#include <sys/poll.h>
#endif

/// \brief This class provides an interface for polling a number of file descriptors.
/// It uses round robin polling
class BPoll {
public:
	typedef struct pollfd PollFd;

			BPoll();
			~BPoll();
			
	void		append(int fd, int events = POLLIN|POLLERR|POLLHUP|POLLNVAL);	///< Append a file descriptor to polling list
	void		delFd(int fd);							///< Remove a file descriptor from polling list

	BError		doPoll(int& fd, int timeoutUs = -1);				///< Perform polling operation
	BError		doPollEvents(int& fd, int& events, int timeoutUs = -1);		///< Perform polling operation and return events

	int		getPollFdsNum();
	PollFd*		getPollFds();
	void		clear();
private:
	int		nextFd(int i);
	
	int		ofdsNum;			///< The number of FD's in list
	PollFd*		ofds;				///< The list of poll fd's
	int		ofdsNext;			///< The next list entry for round robin polling
};

#endif
