/*******************************************************************************
 *	BTask.h	BTask class
 *	T.Barnaby,	Beam Ltd,	2012-11-12
 *	Copyright (c) 2012 All Right Reserved, Beam Ltd, http://www.beam.ltd.uk
 *******************************************************************************
 *
 * Stack size of 0 assumes system will grow using MMU.
 * Task priorities can be set from 0 to 4 inclusive. Basic default and idle priority is 0.
 */
#ifndef BTask_h
#define BTask_h

#include <BError.h>
#include <pthread.h>

class BTask {
public:
			BTask(const char* name = "", BUInt stackSize = 0, BUInt priority = 1);
			~BTask();

	void		init(const char* name, BUInt stackSize = 0, BUInt priority = 1);
	BError		start();		///< Starts the task running
	void		stop();
	void		waitForCompletion();

	// Task manipulation
	int		setPriority(BUInt priority);	///< Set the priority of the task: 0 upwards

	// Task operation
	virtual void	run();			////< The tasks main run loop

protected:
	static void*	taskFunc(void*);
	
	const char*	oname;
	BUInt		ostackSize;
	BUInt		opolicy;
	BUInt		opriority;
	pthread_t	othread;
	Bool		orunning;
};
#endif 
