/*******************************************************************************
 *	BWEditList.h	BWEdit manipulation
 *			T.Barnaby,	BEAM Ltd,	2008-06-02
 *******************************************************************************
 */
#ifndef BWEditList_H
#define BWEditList_H

#include <BWidgets.h>
#include <qlabel.h>
#include <qtablewidget.h>
#include <qdialog.h>
#include <qlayout.h>
#include <qlineedit.h>
#include <qtextedit.h>
#include <qdatetimeedit.h>
#include <qcombobox.h>
#include <qaction.h>
#include <qmenu.h>
#include <qstandarditemmodel.h>
#include <qtreeview.h>

class Gui;
class BWEditList;
enum BWEditAction	{ BWEditView, BWEditEdit, BWEditDelete };

class BWEditObjDialog : public QDialog {
        Q_OBJECT
public:
				BWEditObjDialog(QWidget* parent, BWEditList& editList);
	virtual			~BWEditObjDialog();

	virtual void		setReadOnly(int on);
	
public slots:
	virtual BError		objAction(BWEditAction action, void* obj);
	virtual void		update();
	virtual void		save();
	virtual void		cancel();

	bool			eventFilter(QObject* o, QEvent* e);

protected:
	BWEditList&		oeditList;
	QVBoxLayout		olayout;
	BVBox*			owCenter;
	BHBox*			owButtons;
	QPushButton*		owSaveButton;
	QPushButton*		owCancelButton;
	void*			obj;
};


/// BWEditList manipulation
class BWEditList : public BVBox {
        Q_OBJECT
public:
			        BWEditList(QWidget* w, Gui& gui, int showOptions = 0);
	virtual			~BWEditList();

	Gui&			gui();
#ifdef ZAP
	void			setSelector(Bds::Selection sel);
	Bds::Selection		getSelection();
#endif
	void			setRowCount(int number);
	void			setReadOnly(int on);

	QAction*		optionAddAction(QString name, QString accelKey, QString memberFunc, QString tip);

public slots:
	virtual void		view();
	virtual void		update();
	virtual void		append();
	virtual void		modify();
	virtual void		del();
	virtual void		options();
	virtual BError		objAction(BWEditAction action, int num);

private slots:

protected:
	Gui&			ogui;
	int			omode;
	int			oshowSelector;
	int			oshowOptions;
	int			oreadOnly;
	BVBox*			owSelector;

	BVBox*			owTables;
	BWTableView*		owTable;

	QStandardItemModel*	omodel;
	QTreeView*		owTreeView;

	BHBox*			owButtons;
	BList<QPushButton*>	owButtonList;
	QMenu*			owOptionsMenu;
	BWEditObjDialog*	owEditObjDialog;
#ifdef ZAP
	Bds::Selection		osel;
#endif
};

/// BWEditList manipulation
class BWSelectList : public QDialog {
        Q_OBJECT
public:
			        BWSelectList(QWidget* w, Gui& gui);
	virtual			~BWSelectList();

	Gui&			gui();
	void			setNumRows(int number);
	void			setReadOnly(int on);
	
public slots:
	virtual void		select(int number);
	virtual void		update();

private slots:
	void			select();

protected:
	Gui&			ogui;
	QVBoxLayout		olayout;
	BWTableView*		owTable;
	BHBox*			owButtons;
};
#endif
